VARS = dict(
    # vagrant conf
    proj_ip='10.1.1.123',
    box_name='debian/jessie64',
    vagrant_memory=2048,
    vagrant_cpus=2,

    # include / declude config
    POSTGRES=True,
    CELERY=False,
    REDIS=False,

    # project settings
    proj_name=None,
    root_dir=None,

    # database config
    db_name=None,
    db_pass=None,
    db_user=None,

    # locale conf
    locale='ru_RU',
    encoding='UTF-8',

    # python version
    dj_version='1.9.5',
    py_version='3.5.1',
    pyenv_version=None,
)



#!/usr/bin/env python

import re
import sys
import argparse
import subprocess
import string
import random

from . import __version__
from .settings import VARS
from .local_provision import common

if sys.version_info.major == 3:
    from urllib.request import urlopen
    # allow py3 use input
    raw_input = input
elif sys.version_info.major == 2:
    from urllib import urlopen


def vagrant_fabric():
    """ Check is vagrant fabric installed and if not - install """
    plugins = subprocess.check_output(['vagrant', 'plugin', 'list'])
    if 'vagrant-fabric' not in str(plugins):
        # https://github.com/wutali/vagrant-fabric
        subprocess.call(['vagrant', 'plugin', 'install', 'vagrant-fabric'])


def str_random(size=9, chars=string.ascii_uppercase + string.digits + string.ascii_lowercase):
    """ Not a fabric task, it will generate random str for passw """
    return ''.join(random.choice(chars) for _ in range(size))


def from_user(msg, default, validate, yesno=False):
    """ Get value from user """
    message = '\033[1m> {}\033[0m [{}] '.format(msg, ('y/N', 'Y/n')[default] if yesno else repr(default))
    validator = re.compile(validate)
    if VARS.get('no_input'):
        return default
    while True:
        value = raw_input(message).strip()
        if not value and default is not None:
            return default
        if re.match(validator, value):
            if yesno:
                return value.lower() == 'y'
            return value
        print('\033[31m\033[1m> Incorrect input format\033[0m\033[31m check regex r"{}"\033[0m'.format(validate))


def read_data(args):
    VARS.update(args)

    proj_name = ''.join(re.split(r'[^a-z]', (VARS.get('root_dir') or 'test').lower()))
    VARS['proj_name'] = proj_name or from_user(
        'Enter a project name', proj_name, r'^[a-z]+([a-z]|\d)*$')
    if not VARS.get('root_dir'):
        VARS['root_dir'] = VARS['proj_name']

    VARS['dj_version'] = from_user(
        'Django version', VARS.get('dj_version'), r'^([0-9]{1,2}\.){1,2}[0-9]{1,2}$')

    while True:
        py_version = from_user(
            'Python version', VARS.get('py_version'), r'^([0-9]{1,2}\.){1,2}[0-9]{1,2}$')

        if urlopen('https://www.python.org/ftp/python/{py_version}/'.format(py_version=py_version)).getcode() == 200:
            VARS.update(dict(py_version=py_version, pyenv_version=re.findall(r'^\d{1,2}\.\d{1,2}', py_version)[0]))
            break
        print('\033[31m\033[1m> Python version "{py_version}" not available\033[0m\033[31m '
              'check uri https://www.python.org/ftp/python/\033[0m'.format(py_version=py_version))

    VARS['proj_ip'] = from_user(
        'Vagrant box IP-addr', VARS.get('proj_ip'), r'^([0-9]{1,3}\.){3}[0-9]{1,3}$')

    for name in 'POSTGRES', 'CELERY', 'REDIS':
        VARS[name] = from_user(
            'Do you nead a {}?'.format(name), VARS.get(name, False), r'^[YyNn]{1}$', yesno=True)

    if VARS.get('POSTGRES'):
        for name in ('db_name', 'db_user', 'db_pass'):
            default = '{proj_name}_{}'.format(['db', 'user'][name == 'db_user'], **VARS)
            if name == 'db_pass':
                default = str_random()
            VARS[name] = from_user(
                'Database {}'.format(name).replace('db_', ''), VARS.get(name) or default, r'^\w+$')


def parse():
    parser = argparse.ArgumentParser(description='Falstart is a fast develop template box start tool.')
    parser.add_argument('root_dir', nargs=1, help='Name of started project', type=str)
    parser.add_argument(
        '--no-input', dest='no_input', action='store_true',
        help='Non interactive config parse', default=False)
    parser.add_argument(
        '--box', dest='custom_box', action='store_true',
        help='Pack vagrant to custom box', default=False)
    parser.add_argument(
        '-v', '--version', action='version', version='falstart version {}'.format(__version__))
    result = parser.parse_args().__dict__
    result['root_dir'] = result['root_dir'][0]
    return result


def main():
    parse_data = parse()
    if parse_data['custom_box']:
        common('make_custom_box', parse_data['root_dir'])
    else:
        vagrant_fabric()
        read_data(parse_data)
        common('start_box', parse_data['root_dir'], VARS)



import json
import os
import shutil
import subprocess

from jinja2 import Environment, FileSystemLoader


VARS = dict(
    init_app=True,
    custom_box=False,
    templates_dir=os.path.join(os.path.dirname(__file__), 'templates')
)


def falstart_print(message, error=False, prefix='[falstart] >'):
    """ Display given message with prefix"""
    bold, red, end = '\033[1m', '\033[31m', '\033[0m'
    print(''.join((
        red if error else '',
        bold, prefix, end,
        red if error else '',
        ' ', message, end,
    )).strip())


def run(command):
    """ wrapper of subprocess check_call fun """
    falstart_print('run: "{}"'.format(command))
    subprocess.check_call(command, shell=True)


def put(src, dst):
    falstart_print('copy: "{}" > "{}"'.format(src, dst))
    if os.path.exists(dst):
        shutil.rmtree(dst)
    shutil.copytree(src, dst)


def mkdir(path):
    falstart_print('make dir: "{}"'.format(path))
    try:
        os.makedirs(path)
    except OSError:
        pass


def render_template(template_name, remote_name):
    """ Render wrapper for simplify touch rendered template into target """
    if os.path.dirname(remote_name):
        mkdir(os.path.dirname(remote_name))
    # load jinja template
    jinja_env = Environment(loader=FileSystemLoader(VARS['templates_dir']))
    template = jinja_env.get_template(template_name)
    # write to remote file
    falstart_print('render "{}" > "{}"'.format(template_name, remote_name))
    with open(remote_name, 'w') as target_file:
        target_file.write(template.render(**VARS))


def common(taskname, root_dir, config=None):
    """ Run all common tasks """
    root_dir = os.path.join(os.getcwd(), root_dir)
    mkdir(root_dir)
    os.chdir(root_dir)
    # update configs
    if config is None:
        VARS.update(json.load(open('.falstart.json', 'r')))
    else:
        json.dump(config, open('.falstart.json', 'w'))
        VARS.update(config)
    VARS['root_dir'] = root_dir
    globals()[taskname]()


def start_box():
    """ Create Vagrant file and up virtual machine """
    # render templates for vagrant up
    render_template('Vagrantfile.j2', 'Vagrantfile')
    render_template('Makefile.j2', 'Makefile')
    run('chmod +x Makefile')
    # make provisioning folder
    mkdir('provision')
    render_template('provision_fabfile.j2', 'provision/fabric_provisioner.py')
    render_template('requirements.j2', 'requirements.txt')
    render_template('requirements.j2', 'requirements-remote.txt')
    render_template('settings_local.j2', '{proj_name}/settings_local.py.example'.format(**VARS))
    if VARS.get('CELERY'):
        render_template('celery.j2', '{proj_name}/celery.py'.format(**VARS))
    # copy templates for vagrant fabric render
    put(os.path.join(VARS['templates_dir'], 'vagrant_templates'), 'provision/templates')
    try:
        # run vagrant up
        run('vagrant up')
    except:
        falstart_print('Error occurred when up the vagrant machine.', error=True)
        falstart_print('You can try to manually start the Vagrant.', error=True)

    # replace template
    VARS['init_app'] = False
    render_template('provision_fabfile.j2', 'provision/fabric_provisioner.py')
    # commit changes
    falstart_commit()


def falstart_commit():
    """ Try to commit after box start """
    ignore = (
        '*.py[cod]',
        '__pycache__/',
        '# custom ignore',
        'settings_local.py',
        '.vagrant',
        'var/',
        'static/',
        '',
    )

    falstart_print('Update .gitignore')
    with open('.gitignore', 'a') as gitignore:
        gitignore.write('\n'.join(ignore))

    for attempt in range(2):
        try:
            run('git add . && git commit -m ":rocket: falstart init commit"')
            break
        except:
            falstart_print('not a git repository', error=True)
            run('git init')


def make_custom_box():
    VARS['custom_box'] = True
    run('vagrant destroy -f')
    # render templates to no provide app and syncfolder
    render_template('Vagrantfile.j2', 'Vagrantfile')
    run('vagrant up')
    for cmd in (
            'sudo dd if=/dev/zero of=/EMPTY bs=1M', 'sudo rm -f /EMPTY',
            'cat /dev/null > ~/.bash_history && history -c'):
        run('vagrant ssh -c "{}"'.format(cmd))
    # make custom box
    run('vagrant package --output ~/{proj_name}.box'.format(**VARS))
    # return template to back
    VARS['custom_box'] = False
    render_template('Vagrantfile.j2', 'Vagrantfile')


def rmproj():
    """ Remove project """
    run('cd {root_dir} && vagrant destroy'.format(**VARS))
    run('rm {root_dir} -rf'.format(**VARS))



# -*- coding: utf-8 -*-

import io
import os
import re

from setuptools import setup


version = re.search('^__version__\s*=\s*"(.*)"', open('falstart/__init__.py').read(), re.M).group(1)


def read(fname):
    return io.open(os.path.join(os.path.dirname(__file__), fname), encoding='UTF-8').read()


with open('requirements.txt') as requf:
    required = requf.read().splitlines()


setup(
    name='falstart',
    version=version,
    author='Maks Skorokhod',
    author_email='samae500@gmail.com',
    url='https://github.com/Samael500/falstart',
    packages=['falstart'],
    package_data={
        'falstart': [
            'templates/*.j2',
            'templates/vagrant_templates/*.j2',
        ]
    },
    license='MIT',
    description='Tool for fast start develop box template.',
    long_description=read('README.rst'),
    install_requires=required,
    data_files=[('', ['LICENSE', 'README.rst', ]), ],
    entry_points={
        'console_scripts': ['falstart = falstart.falstart:main']
    },
    classifiers=[
        'Environment :: Console',
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    keywords='falstart fabric vagrant virtualbox development fast start template'.split(),
)
